
import os
from google.auth.transport.requests import Request
from google.oauth2.credentials import Credentials
from google_auth_oauthlib.flow import InstalledAppFlow
from pathlib import Path

# source: https://developers.google.com/identity/protocols/oauth2/web-server#obtainingaccesstokens

# ===
# The following steps show how your application interacts with Google's OAuth 2.0 server to obtain 
# a user's consent to perform an API request on the user's behalf. 
# Your application must have that consent before it can execute a Google API request that 
# requires user authorization.
#
# The list below quickly summarizes these steps:
# 
# 1. Your application identifies the permissions it needs.
# 2. Your application redirects the user to Google along with the list of requested permissions.
# 3. The user decides whether to grant the permissions to your application.
# 4. Your application finds out what the user decided.
# 5. If the user granted the requested permissions, your application retrieves tokens needed to make 
#    API requests on the user's behalf.
# ===

# source: https://developers.google.com/identity/protocols/oauth2/scopes#sheets
# See, edit, create and delete
SCOPES = ['https://www.googleapis.com/auth/spreadsheets'] 

class Auth:
    """
    The Class `Auth` will authenticate the current user with Google's account
    """

    def get(self) -> object:

        creds = None
        cwd = os.getcwd()

        # TODO [dev]: replace the code below with using the GCP Secret Manager to store both credentials and token files
        (os.chdir(path=str(Path.home()) + '/crgoauth'))

        if os.path.exists('token.json'):
            creds = Credentials.from_authorized_user_file('token.json', SCOPES)
        # If there are no (valid) credentials available, let the user log in.
        if not creds or not creds.valid:
            if creds and creds.expired and creds.refresh_token:
                creds.refresh(Request())
            else:
                flow = InstalledAppFlow.from_client_secrets_file(
                    'credentials.json', SCOPES)
                creds = flow.run_local_server(port=0)
            # Save the credentials for the next run
            with open('token.json', 'w') as token:
                token.write(creds.to_json())

        os.chdir(cwd)
        if creds is None:
            return None
        return creds



from __future__ import print_function

import os.path
import datetime
import fore_cloudreach as fc 


# The ID and range of a sample spreadsheet.
SAMPLE_SPREADSHEET_ID = '1Ni5B7zolPKD2J0CXQSaLec31AXw8HpcdM_xPJKPILRw' #'1BxiMVs0XRA5nFMdKvBdBZjgmUUqptlbs74OgvE2upms'
SAMPLE_RANGE_NAME = 'Rechnung!F5:G8'

def read_test(spreadsheet_id):
    """ Show basic usage of the Sheets API.
        Prints values from a sample spreadsheet
    """
    authet = fc.Auth() 
    creds = authet.get()

    if creds is None:
        raise Exception("invalid user authentication")
        return

    if spreadsheet_id == '':
        spd_id = input("Enter the Google Sheet file ID from the URL:\n")
        spreadsheet_id = spd_id 

    print("currentDir: ", format(os.getcwd()))

    tmp = fc.Template(template_id=spreadsheet_id)
    print(f"processing file id: {tmp.id} ...")

    month_year = datetime.datetime.now().strftime('%m-%Y')
    resp = tmp.new_from_template(creds, spreadsheet_id, month_year)
    print("\n\n\nresp:\n\n", format(resp))